#
# Copyright 2020 Ettus Research, a National Instruments Brand
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
"""
Access to the property tree
"""

from . import libpyuhd as lib

PropertyTree = lib.property_tree
