



uhd_cal_rx_iq_balance --help || if !ERRORLEVEL! LEQ 1 cmd /c "exit /b 0"
IF %ERRORLEVEL% NEQ 0 exit /B 1
uhd_cal_tx_dc_offset --help || if !ERRORLEVEL! LEQ 1 cmd /c "exit /b 0"
IF %ERRORLEVEL% NEQ 0 exit /B 1
uhd_cal_tx_iq_balance --help || if !ERRORLEVEL! LEQ 1 cmd /c "exit /b 0"
IF %ERRORLEVEL% NEQ 0 exit /B 1
uhd_config_info --help || if !ERRORLEVEL! LEQ 1 cmd /c "exit /b 0"
IF %ERRORLEVEL% NEQ 0 exit /B 1
uhd_find_devices --help || if !ERRORLEVEL! LEQ 1 cmd /c "exit /b 0"
IF %ERRORLEVEL% NEQ 0 exit /B 1
uhd_image_loader --help || if !ERRORLEVEL! LEQ 1 cmd /c "exit /b 0"
IF %ERRORLEVEL% NEQ 0 exit /B 1
uhd_images_downloader --help || if !ERRORLEVEL! LEQ 1 cmd /c "exit /b 0"
IF %ERRORLEVEL% NEQ 0 exit /B 1
uhd_usrp_probe --help || if !ERRORLEVEL! LEQ 1 cmd /c "exit /b 0"
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\\Library\\bin\\uhd.dll exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\\Library\\lib\\uhd.lib exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
uhd_config_info --print-all
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
